<?php

namespace Drupal\pimple_container_example\importer;

abstract class Importer {

  /**
   * Data source container.
   */
  protected $dataSource;

  /**
   * Data destination container.
   */
  protected $dataDestination;

  /**
   * Data processor container.
   */
  protected $dataProcessor;

  /**
   * Data container.
   */
  protected $data;

  /**
   * Constructor.
   *
   * @param DataSource $data_source
   *   Data source of SPC data.
   *
   * @param DataDestination $data_destination
   *   Destination for SPC data.
   *
   * @param DataProcessor $data_processor
   *   Destination for SPC data.
   */
  public function __construct(DataSource $data_source, DataDestination $data_destination, DataProcessor $data_processor) {
    $this->dataSource = $data_source;
    $this->dataDestination = $data_destination;
    $this->dataProcessor = $data_processor;
  }

  /**
   * Callback executed after import finished.
   */
  protected abstract function afterImport();

  /**
   * Import data.
   */
  public function import() {
    if ($data = $this->dataSource->getData()) {
      if ($processed_data = $this->dataProcessor->processData($data)) {
        $this->dataDestination->saveData($processed_data);
        $this->data = $processed_data;
        $this->afterImport();
      }
    }
  }

  /**
   * Set data processor.
   */
  public function setDataProcessor($data_processor) {
    $this->dataProcessor = $data_processor;
  }

}
